﻿using Connected.Models;
using Connected.Utilities;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using System.Numerics;
using static Connected.Colors;

namespace Connected.Components;
public partial class NumberInput<NumberType>:InputBase where NumberType : INumber<NumberType>
{
	private double _step =1;
	[Parameter]
	public double Step { 
		get
		{
			return _step;
		}
		set
		{
			_step=value;
		}
	}

	[Parameter]
	public bool DisableMouseWheel { get; set; } = false;

	private async Task StepUp()
	{
		double num = (double)Convert.ChangeType(Value, typeof(double));
		num += _step;
		Value = AdjustDecimalPlaces((NumberType)Convert.ChangeType(num, typeof(NumberType)));
		await ValueChanged.InvokeAsync(Value);
	}

	private async Task StepDown()
	{
		double num = (double)Convert.ChangeType(Value, typeof(double));
		num -= _step;
		Value = AdjustDecimalPlaces((NumberType)Convert.ChangeType(num, typeof(NumberType)));
		await ValueChanged.InvokeAsync(Value);
	}
	protected async Task OnMouseWheel(WheelEventArgs obj)
	{
		if (DisableMouseWheel==false)
		{
			if (!obj.ShiftKey || Disabled || Readonly)
				return;
			if (obj.DeltaY < 0)
			{
				await StepDown();
			}
			else
			{
				await StepUp();
			}
		}
	}


	[Parameter]
	public NumberType Value
	{
		get;
		set;
	}

	[Parameter]
	public int DecimalPlaces { get; set; } =0;


	[Parameter]
    public EventCallback<NumberType> ValueChanged { get; set; }

	public async Task ChangeValueAsync(ChangeEventArgs args)
    {
		if (args.Value is not null)
		{
				NumberType value = (NumberType)Convert.ChangeType(args.Value, typeof(NumberType));
				if (value.ToString().Length <= 0)
				{

					value = (NumberType)Convert.ChangeType(0, typeof(NumberType));
				}
				await ValueChanged.InvokeAsync((NumberType)Convert.ChangeType(AdjustDecimalPlaces(value), typeof(NumberType)));
		}
	}

	private NumberType AdjustDecimalPlaces(NumberType value)
	{
		NumberType result = value;
		if (Helper.NumberHasDecimalPlaces(value))
		{
			if (DecimalPlaces > 0)
			{
				decimal rounded = (decimal)Convert.ChangeType(value, typeof(decimal));
				rounded = Math.Round(rounded, DecimalPlaces);
				result = (NumberType)Convert.ChangeType(rounded, typeof(NumberType));
			}
		}
		return result;
	}

	private async Task Clear()
	{
		await ValueChanged.InvokeAsync((NumberType)Convert.ChangeType(0, typeof(NumberType)));
	}

	#region Lifecycle

	protected override async Task OnInitializedAsync()
	{
		if (DecimalPlaces > 0)
		{
			Value = AdjustDecimalPlaces(Value);
			await ValueChanged.InvokeAsync(Value);
		}
	}

	protected override async Task OnParametersSetAsync()
	{
		if (typeof(NumberType).Name.ToLower().Contains("int"))
		{
			if (Step - (int)Step > 0) Step = (int)Step;
			if (Step < 1) Step = 1;
		}
		await base.OnParametersSetAsync();
    }

    #endregion
}